@@ -33,5 +33,6 @@ public interface VersionService {
     Path getPathToPreviousVersion(String uri);
 
     Path restore(final Path path,
-                 final String comment);
+                 final String comment,
+                 final String branchName);
 }
