@@ -68,11 +68,13 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
                 break;
 
             case COLUMN_MOVE:
+            case COLUMN_MOVE_INITIATED:
                 //If we're currently moving a column we don't need to find a column
                 handleColumnMove(event);
                 break;
 
             case ROW_MOVE:
+            case ROW_MOVE_INITIATED:
                 //If we're currently moving a row we don't need to find a row
                 handleRowMove(event);
                 break;
@@ -139,6 +141,7 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
             if (cy < headerMaxY) {
                 //Check for column moving
                 findMovableColumns(gridWidget,
+                                   renderingInformation,
                                    headerHeight - headerRowsYOffset,
                                    headerMinY,
                                    cx,
@@ -146,14 +149,16 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
             } else {
                 //Check for movable rows
                 findMovableRows(gridWidget,
+                                renderingInformation,
                                 cx,
                                 cy);
-
-                //Check for column resizing
-                findResizableColumn(gridWidget,
-                                    cx);
             }
 
+            //Check for column resizing
+            findResizableColumn(gridWidget,
+                                renderingInformation,
+                                cx);
+
             //If over Grid but no operation has been identified default to Grid dragging; when unpinned
             if (!layer.isGridPinned()) {
                 if (state.getActiveGridWidget() == null) {
@@ -168,7 +173,7 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
         }
     }
 
-    private void setCursor(final Style.Cursor cursor) {
+    protected void setCursor(final Style.Cursor cursor) {
         for (IMediator mediator : layer.getViewport().getMediators()) {
             if (mediator instanceof RestrictedMousePanMediator) {
                 if (((RestrictedMousePanMediator) mediator).isDragging()) {
@@ -181,10 +186,9 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
     }
 
     protected void findResizableColumn(final GridWidget view,
+                                       final BaseGridRendererHelper.RenderingInformation renderingInformation,
                                        final double cx) {
         //Gather information on columns
-        final BaseGridRendererHelper rendererHelper = view.getRendererHelper();
-        final BaseGridRendererHelper.RenderingInformation renderingInformation = rendererHelper.getRenderingInformation();
         if (renderingInformation == null) {
             return;
         }
@@ -243,13 +247,12 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
     }
 
     protected void findMovableColumns(final GridWidget view,
+                                      final BaseGridRendererHelper.RenderingInformation renderingInformation,
                                       final double headerRowsHeight,
                                       final double headerMinY,
                                       final double cx,
                                       final double cy) {
         //Gather information on columns
-        final BaseGridRendererHelper rendererHelper = view.getRendererHelper();
-        final BaseGridRendererHelper.RenderingInformation renderingInformation = rendererHelper.getRenderingInformation();
         if (renderingInformation == null) {
             return;
         }
@@ -327,25 +330,27 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
     }
 
     protected void findMovableRows(final GridWidget view,
+                                   final BaseGridRendererHelper.RenderingInformation renderingInformation,
                                    final double cx,
                                    final double cy) {
-        if (!isOverRowDragHandleColumn(view,
+        if (!isOverRowDragHandleColumn(renderingInformation,
                                        cx)) {
             return;
         }
 
         final GridData gridModel = view.getModel();
         final GridRenderer renderer = view.getRenderer();
+        final List<Double> allRowHeights = renderingInformation.getAllRowHeights();
 
         //Get row index
         if (gridModel.getRowCount() == 0) {
             return;
         }
-        GridRow row;
+        double rowHeight;
         int uiRowIndex = 0;
         double offsetY = cy - renderer.getHeaderHeight();
-        while ((row = gridModel.getRow(uiRowIndex)).getHeight() < offsetY) {
-            offsetY = offsetY - row.getHeight();
+        while ((rowHeight = allRowHeights.get(uiRowIndex)) < offsetY) {
+            offsetY = offsetY - rowHeight;
             uiRowIndex++;
         }
         if (uiRowIndex < 0 || uiRowIndex > gridModel.getRowCount() - 1) {
@@ -353,7 +358,7 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
         }
 
         //Add row over which MouseEvent occurred
-        final List<GridRow> rows = new ArrayList<GridRow>();
+        final List<GridRow> rows = new ArrayList<>();
         rows.add(gridModel.getRow(uiRowIndex));
 
         //Add any other collapsed rows
@@ -369,11 +374,9 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
         setCursor(Style.Cursor.MOVE);
     }
 
-    private boolean isOverRowDragHandleColumn(final GridWidget view,
+    private boolean isOverRowDragHandleColumn(final BaseGridRendererHelper.RenderingInformation renderingInformation,
                                               final double cx) {
         //Gather information on columns
-        final BaseGridRendererHelper rendererHelper = view.getRendererHelper();
-        final BaseGridRendererHelper.RenderingInformation renderingInformation = rendererHelper.getRenderingInformation();
         if (renderingInformation == null) {
             return false;
         }
@@ -468,13 +471,16 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
             if (rightGridColumn.isPresent()) {
                 GridColumn<?> rightColumn = rightGridColumn.get();
                 double originalRightColumnWidth = rightColumn.getWidth();
-                double newWidth = originalRightColumnWidth + delta;
+                double widthWithoutColumn = gridWidgetWidth - originalRightColumnWidth;
+
+                double newWidth = visibleWidth - widthWithoutColumn + delta;
 
                 rightColumn.setWidth(newWidth);
             }
             // or revert column resizing if the column itself has AUTO width
             else if (GridColumn.ColumnWidthMode.isAuto(activeGridColumn)) {
-                columnNewWidth = originalLeftColumnWidth;
+                double widthWithoutColumn = gridWidgetWidth - originalLeftColumnWidth;
+                columnNewWidth = visibleWidth - widthWithoutColumn;
             }
         }
         return columnNewWidth;
@@ -554,6 +560,7 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
                                     activeGridModel.moveColumnsTo(candidateBlockEndColumnIndex,
                                                                   activeGridColumns);
                                     state.getEventColumnHighlight().setX(activeGridWidget.getComputedLocation().getX() + rendererHelper.getColumnOffset(activeGridColumns.get(0)));
+                                    state.setOperation(GridWidgetDnDHandlersState.GridWidgetHandlersOperation.COLUMN_MOVE);
                                     layer.batch();
                                     return;
                                 } else {
@@ -561,6 +568,7 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
                                     activeGridModel.moveColumnsTo(candidateBlockStartColumnIndex,
                                                                   activeGridColumns);
                                     state.getEventColumnHighlight().setX(activeGridWidget.getComputedLocation().getX() + rendererHelper.getColumnOffset(activeGridColumns.get(0)));
+                                    state.setOperation(GridWidgetDnDHandlersState.GridWidgetHandlersOperation.COLUMN_MOVE);
                                     layer.batch();
                                     return;
                                 }
@@ -636,11 +644,11 @@ public class GridWidgetDnDMouseMoveHandler implements NodeMouseMoveHandler {
 
         //Move row(s) and update highlight
         destroyColumns(allGridColumns);
-        activeGridModel.moveRowsTo(uiRowIndex,
-                                   activeGridRows);
+        activeGridModel.moveRowsTo(uiRowIndex, activeGridRows);
 
         final double rowOffsetY = rendererHelper.getRowOffset(leadRow) + headerHeight;
         state.getEventColumnHighlight().setY(activeGridWidget.getComputedLocation().getY() + rowOffsetY);
+        state.setOperation(GridWidgetDnDHandlersState.GridWidgetHandlersOperation.ROW_MOVE);
         layer.batch();
     }
 
