@@ -20,16 +20,14 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.Objects;
 import java.util.function.BiFunction;
-import java.util.stream.IntStream;
 
-import com.ait.lienzo.client.core.shape.BoundingBoxPathClipper;
 import com.ait.lienzo.client.core.shape.Group;
 import com.ait.lienzo.client.core.shape.IPathClipper;
 import com.ait.lienzo.client.core.shape.MultiPath;
 import com.ait.lienzo.client.core.shape.Rectangle;
-import com.ait.lienzo.client.core.types.BoundingBox;
 import com.ait.lienzo.client.core.types.Transform;
 import com.ait.lienzo.shared.core.types.ColorName;
+import com.google.gwt.core.client.GWT;
 import org.uberfire.ext.wires.core.grids.client.model.GridCell;
 import org.uberfire.ext.wires.core.grids.client.model.GridColumn;
 import org.uberfire.ext.wires.core.grids.client.model.GridData;
@@ -57,6 +55,7 @@ public class ColumnRenderingStrategyMerged {
         final double clipMinX = context.getClipMinX();
         final int minVisibleRowIndex = context.getMinVisibleRowIndex();
         final int maxVisibleRowIndex = context.getMaxVisibleRowIndex();
+        final List<Double> allRowHeights = renderingInformation.getAllRowHeights();
         final List<Double> visibleRowOffsets = renderingInformation.getVisibleRowOffsets();
         final boolean isFloating = context.isFloating();
         final GridData model = context.getModel();
@@ -64,7 +63,7 @@ public class ColumnRenderingStrategyMerged {
         final GridRenderer renderer = context.getRenderer();
         final GridRendererTheme theme = renderer.getTheme();
         final double columnWidth = column.getWidth();
-        final double columnHeight = visibleRowOffsets.get(maxVisibleRowIndex - minVisibleRowIndex) - visibleRowOffsets.get(0) + model.getRow(maxVisibleRowIndex).getHeight();
+        final double columnHeight = visibleRowOffsets.get(maxVisibleRowIndex - minVisibleRowIndex) - visibleRowOffsets.get(0) + allRowHeights.get(maxVisibleRowIndex);
         final int columnIndex = model.getColumns().indexOf(column);
 
         final List<GridRenderer.RendererCommand> commands = new ArrayList<>();
@@ -115,7 +114,8 @@ public class ColumnRenderingStrategyMerged {
         //Column content
         commands.add((GridRenderer.RenderBodyGridContentCommand) (rc) -> {
             if (columnRenderingConstraint.apply(rc.isSelectionLayer(), column)) {
-                final Group columnGroup = new Group().setX(x);
+                final Group columnGroup = GWT.create(Group.class);
+                columnGroup.setX(x);
                 int iterations = 0;
                 for (int rowIndex = minVisibleRowIndex; rowIndex <= maxVisibleRowIndex; rowIndex++) {
 
@@ -146,7 +146,7 @@ public class ColumnRenderingStrategyMerged {
 
                     if (isCollapsedCellMixedValue) {
                         final Group mixedValueGroup = renderMergedCellMixedValueHighlight(columnWidth,
-                                                                                          row.getHeight());
+                                                                                          allRowHeights.get(rowIndex));
                         mixedValueGroup.setX(0).setY(y).setListening(true);
                         columnGroup.add(mixedValueGroup);
                     }
@@ -162,7 +162,7 @@ public class ColumnRenderingStrategyMerged {
                                                                       columnIndex);
                         if (nextRowCell != null) {
                             final Group gt = renderGroupedCellToggle(columnWidth,
-                                                                     row.getHeight(),
+                                                                     allRowHeights.get(rowIndex),
                                                                      nextRowCell.isCollapsed());
                             gt.setX(0).setY(y);
                             columnGroup.add(gt);
@@ -172,7 +172,7 @@ public class ColumnRenderingStrategyMerged {
                     if (cell.getMergedCellCount() > 0) {
                         //If cell is "lead" i.e. top of a merged block centralize content in cell
                         final double cellHeight = getCellHeight(rowIndex,
-                                                                model,
+                                                                allRowHeights,
                                                                 cell);
                         final GridBodyCellRenderContext cellContext = new GridBodyCellRenderContext(absoluteColumnX,
                                                                                                     absoluteGridY + renderer.getHeaderHeight() + visibleRowOffsets.get(rowIndex - minVisibleRowIndex),
@@ -201,14 +201,14 @@ public class ColumnRenderingStrategyMerged {
                         GridCell<?> _cell = cell;
                         while (_cell.getMergedCellCount() == 0) {
                             _rowIndex--;
-                            _y = _y - model.getRow(_rowIndex).getHeight();
+                            _y = _y - allRowHeights.get(_rowIndex);
                             _cell = model.getCell(_rowIndex,
                                                   columnIndex);
                         }
 
                         final double cellHeight = getCellHeight(_rowIndex,
-                                                                model,
-                                                                _cell);
+                                                                allRowHeights,
+                                                                cell);
                         final GridBodyCellRenderContext cellContext = new GridBodyCellRenderContext(absoluteColumnX,
                                                                                                     absoluteGridY + renderer.getHeaderHeight() + rendererHelper.getRowOffset(_rowIndex),
                                                                                                     columnWidth,
@@ -234,11 +234,11 @@ public class ColumnRenderingStrategyMerged {
 
                 //Clip Column Group
                 final double gridLinesStrokeWidth = theme.getBodyGridLine().getStrokeWidth();
-                final BoundingBox bb = new BoundingBox(gridLinesStrokeWidth,
-                                                       0,
-                                                       columnWidth - gridLinesStrokeWidth,
-                                                       columnHeight);
-                final IPathClipper clipper = new BoundingBoxPathClipper(bb);
+                final BoundingBoxPathClipperFactory boundingBoxPathClipperFactory = GWT.create(BoundingBoxPathClipperFactory.class);
+                final IPathClipper clipper = boundingBoxPathClipperFactory.newClipper(gridLinesStrokeWidth,
+                                                                                      0,
+                                                                                      columnWidth - gridLinesStrokeWidth,
+                                                                                      columnHeight);
                 columnGroup.setPathClipper(clipper);
                 clipper.setActive(true);
 
@@ -305,11 +305,13 @@ public class ColumnRenderingStrategyMerged {
     }
 
     protected static double getCellHeight(final int rowIndex,
-                                          final GridData model,
+                                          final List<Double> allRowHeights,
                                           final GridCell<?> cell) {
-        return IntStream.range(rowIndex, rowIndex + cell.getMergedCellCount())
-                .mapToDouble(index -> model.getRow(index).getHeight())
-                .sum();
+        double height = 0.0;
+        for (int iRowIndex = rowIndex; iRowIndex < rowIndex + cell.getMergedCellCount(); iRowIndex++) {
+            height = height + allRowHeights.get(iRowIndex);
+        }
+        return height;
     }
 
     private static Group renderGroupedCellToggle(final double cellWidth,
